/************************************************************************
	interrupt.c

    Simon18F - An MB Electronics Simon clone based on the PIC18F2550
    Copyright (C) 2009 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#include <htc.h>

#include "interrupt.h"
#include "hardware.h"
#include "spi.h"

// Low priority interrupt procedure
void interrupt low_priority lpHandler(void)
{
	unsigned char hByte = 0x00;
	unsigned char lByte = 0x00;
	unsigned char light;

	// Is this timer0 interrupting?
	if (TMR0IF)
	{
		// Write the state of the lights **********************************************************
		for (light = 0; light < 12; light++)
		{
			if ((hardware.light[light].brightness == 100) ||
				(hardware.light[light].brightness != 0 && hardware.pwmCounter < hardware.light[light].brightness))
			{
				// This switch translates the LED identifier to the physical LED port
				switch(light)
				{
					case(REDL):		hByte |= 0b00000000; lByte |= 0b00000100;
									break;
					case(REDC):		hByte |= 0b00000000; lByte |= 0b00000010;
									break;
					case(REDR):		hByte |= 0b00000000; lByte |= 0b00000001;
									break;
					case(GREENL):	hByte |= 0b00000000; lByte |= 0b00001000;
									break;
					case(GREENC):	hByte |= 0b00000000; lByte |= 0b00010000;
									break;
					case(GREENR):	hByte |= 0b00000000; lByte |= 0b00100000;
									break;
					case(YELLOWL):	hByte |= 0b00000001; lByte |= 0b00000000;
									break;
					case(YELLOWC):	hByte |= 0b00000000; lByte |= 0b10000000;
									break;
					case(YELLOWR):	hByte |= 0b00000000; lByte |= 0b01000000;
									break;
					case(BLUEL):	hByte |= 0b00001000; lByte |= 0b00000000;
									break;
					case(BLUEC):	hByte |= 0b00000100; lByte |= 0b00000000;
									break;
					case(BLUER):	hByte |= 0b00000010; lByte |= 0b00000000;
									break;
				}
			}
		
			// If light is OFF and brightness > 0 then fade the light at dimSpeed
			if ((hardware.light[light].state == OFF) && (hardware.light[light].brightness > 0))
			{
				if (hardware.dimCounter == 0)
				{
					hardware.light[light].brightness -= hardware.light[light].dimSpeed;
					if (hardware.light[light].brightness < 0) hardware.light[light].brightness = 0;
				}
			}
		}

		// Write the state of the LEDs to the serial LED controller
		SPIwrite(hByte, lByte);

		// Increment the PWM counter
		hardware.pwmCounter++;
		if (hardware.pwmCounter == 100) hardware.pwmCounter = 0;

		// Increment the dimming counter
		hardware.dimCounter++;
		if (hardware.dimCounter == 100) hardware.dimCounter = 0;

		// Read the state of the buttons **********************************************************
		if (hardware.buttonCounter == 0)
		{
			// Read the state of the start button and skill level slider
			if (IN1 == ON || IN2 == ON || IN3 == ON || IN4 == ON)
			{
				if (hardware.startButton.state == OFF)
				{
					// If the debounce tolerance is met change state otherwise
					// increment the debounce counter
					if (hardware.startButton.debounceCount > DEBOUNCE_TOLERANCE)
					{
						// Start button is on
						hardware.startButton.state = ON;

						// Now determine the skill level setting
						if(IN1 == ON) hardware.skillSwitch.position = 1;
						if(IN3 == ON) hardware.skillSwitch.position = 2;
						if(IN2 == ON) hardware.skillSwitch.position = 3;
						if(IN4 == ON) hardware.skillSwitch.position = 4;
					}
					else hardware.startButton.debounceCount++;
				}
			}
			else
			{
				// Button is off
				hardware.startButton.state = OFF;
				hardware.startButton.debounceCount = 0;
				//hardware.startButton.autoOffCount = 0;
			}

			// Increment the random number counter
			if (game.randomNumber == 3) game.randomNumber = 0;
			else game.randomNumber++;

			// Set the ports ready for the next read
			START_SKILL = OFF;
			LAST_LONGEST_GAME = ON;
			COLOUR_BUTTONS = OFF;
		}

		if (hardware.buttonCounter == 30)
		{
			// Read the state of the last button, longest button and the game slider

			// NOTE: This code is not good; since we will want to keep the light on
			//   whilst the button is pressed we should debounce both the ON transisition
			//   and the OFF transistion.

			// Last
			if (IN4 == ON)
			{
				if (hardware.lastButton.state == OFF)
				{
					// If the debounce tolerance is met change state otherwise
					// increment the debounce counter
					if (hardware.lastButton.debounceCount > DEBOUNCE_TOLERANCE)
						hardware.lastButton.state = ON;
					else hardware.lastButton.debounceCount++;
				}
			}
			else
			{
				// Button is off
				hardware.lastButton.state = OFF;
				hardware.lastButton.debounceCount = 0;
			}
		
			// Longest
			if (IN3 == ON)
			{
				if (hardware.longestButton.state == OFF)
				{
					// If the debounce tolerance is met change state otherwise
					// increment the debounce counter
					if (hardware.longestButton.debounceCount > DEBOUNCE_TOLERANCE)
						hardware.longestButton.state = ON;
					else hardware.longestButton.debounceCount++;
				}
			}
			else
			{
				// Button is off
				hardware.longestButton.state = OFF;
				hardware.longestButton.debounceCount = 0;
			}
		
			// Game
			if (IN1 == OFF && IN2 == ON ) hardware.gameSwitch.position = 3;
			else if (IN1 == ON  && IN2 == OFF) hardware.gameSwitch.position = 2;
			else hardware.gameSwitch.position = 1;
			

			// Set the ports ready for the next read
			START_SKILL = OFF;
			LAST_LONGEST_GAME = OFF;
			COLOUR_BUTTONS = ON;
		}

		if (hardware.buttonCounter == 60)
		{
			// Read the state of the colour buttons
			unsigned char button;
			unsigned char input;

			for (button = 0; button < 4; button++)
			{
				switch (button)
				{
					case 0:	input = IN1;	// Blue
							break;
					case 1: input = IN2;	// Yellow
							break;
					case 2:	input = IN3;	// Red
							break;
					case 3:	input = IN4;	// Green
							break;
				}

				// Debounce ON
				if (input == ON && hardware.colourButton[button].state == OFF)
				{
					// If the debounce tolerance is met change state otherwise
					// increment the debounce counter
					if (hardware.colourButton[button].debounceCount > DEBOUNCE_TOLERANCE)
						hardware.colourButton[button].state = ON;
					else hardware.colourButton[button].debounceCount++;
				}
	
				// Debounce off
				if (input == OFF && hardware.colourButton[button].state == ON)
				{
					// If the debounce tolerance is met change state otherwise
					// increment the debounce counter
					if (hardware.colourButton[button].debounceCount > DEBOUNCE_TOLERANCE)
						hardware.colourButton[button].state = OFF;
					else hardware.colourButton[button].debounceCount++;
				}
	
				// Reset debounce counter
				if (input == hardware.colourButton[button].state) hardware.colourButton[button].debounceCount = 0;
			}

			// Set the ports ready for the next read
			START_SKILL = ON;
			LAST_LONGEST_GAME = OFF;
			COLOUR_BUTTONS = OFF;
		}

		// Update the button counter
		hardware.buttonCounter++;
		if (hardware.buttonCounter == 90) hardware.buttonCounter = 0;

		// Increment the delay counter
		hardware.delayCounter++;
		if (hardware.delayCounter > 4000000000) hardware.delayCounter = 0;

		// Set the volume level
		if (hardware.volumeLevel != hardware.currentVolumeLevel) setVolumeLevel();

		TMR0L = TMR0_SPEED;	// Reset the timer0 counter
		TMR0IF = 0;			// Clear the timer0 interrupt flag
	}
}

// High priority interrupt procedure
void interrupt hpHandler(void)
{
	// Is this timer1 interrupting? (PWM sound generation interrupt)
	if (TMR1IF)
	{
		// Pulse on or pulse off?
		if (hardware.sound.pwmState == OFF)
		{
			SPEAKER = 1;
			hardware.sound.pwmState = ON;
		}
		else
		{
			SPEAKER = 0;
			hardware.sound.pwmState = OFF;
		}

		// Reset the timer1 counter bytes
		TMR1H = hardware.sound.timer1HighByte;
		TMR1L = hardware.sound.timer1LowByte;

		// Reset the timer1 interrupt flag
		TMR1IF = 0;
	}
}
