/************************************************************************
	lights.c

    Simon18F - An MB Electronics Simon clone based on the PIC18F2550
    Copyright (C) 2009 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

// Global includes
#include <htc.h>

// Local includes
#include "hardware.h"
#include "lights.h"

// These functions are simple utility routines for turning the 
// lights on and off.  They are included to make it easier
// to alter the light control in the future, but don't do
// anything clever right now except allowing the light dim
// speed and brightness to be altered globally.

// This routine is for controlling a single LED
void lightOn(unsigned char light)
{
	hardware.light[light].dimSpeed = 4;
	hardware.light[light].brightness = 100;
	hardware.light[light].state = ON;
}

// This routine is for controlling a single LED
void lightOff(unsigned char light)
{
	hardware.light[light].state = OFF;
}

void lightOffNoFade(unsigned char light)
{
	hardware.light[light].dimSpeed = 100;
	hardware.light[light].state = OFF;
}

// The 'colour' routines operate all 3 LEDs for each colour
void colourOn(unsigned char light)
{
	if (light == RED)
	{
		hardware.light[REDL].dimSpeed = 16;
		hardware.light[REDL].brightness = 100;
		hardware.light[REDL].state = ON;
	
		hardware.light[REDC].dimSpeed = 6;
		hardware.light[REDC].brightness = 100;
		hardware.light[REDC].state = ON;
	
		hardware.light[REDR].dimSpeed = 16;
		hardware.light[REDR].brightness = 100;
		hardware.light[REDR].state = ON;
	}

	if (light == GREEN)
	{
		hardware.light[GREENL].dimSpeed = 16;
		hardware.light[GREENL].brightness = 100;
		hardware.light[GREENL].state = ON;
	
		hardware.light[GREENC].dimSpeed = 6;
		hardware.light[GREENC].brightness = 100;
		hardware.light[GREENC].state = ON;
	
		hardware.light[GREENR].dimSpeed = 16;
		hardware.light[GREENR].brightness = 100;
		hardware.light[GREENR].state = ON;
	}

	if (light == YELLOW)
	{
		hardware.light[YELLOWL].dimSpeed = 16;
		hardware.light[YELLOWL].brightness = 100;
		hardware.light[YELLOWL].state = ON;
	
		hardware.light[YELLOWC].dimSpeed = 6;
		hardware.light[YELLOWC].brightness = 100;
		hardware.light[YELLOWC].state = ON;
	
		hardware.light[YELLOWR].dimSpeed = 16;
		hardware.light[YELLOWR].brightness = 100;
		hardware.light[YELLOWR].state = ON;
	}

	if (light == BLUE)
	{
		hardware.light[BLUEL].dimSpeed = 16;
		hardware.light[BLUEL].brightness = 100;
		hardware.light[BLUEL].state = ON;
	
		hardware.light[BLUEC].dimSpeed = 6;
		hardware.light[BLUEC].brightness = 100;
		hardware.light[BLUEC].state = ON;
	
		hardware.light[BLUER].dimSpeed = 16;
		hardware.light[BLUER].brightness = 100;
		hardware.light[BLUER].state = ON;
	}
}

void colourOff(unsigned char light)
{
	if (light == RED)
	{
		hardware.light[REDL].state = OFF;
		hardware.light[REDC].state = OFF;
		hardware.light[REDR].state = OFF;
	}

	if (light == GREEN)
	{
		hardware.light[GREENL].state = OFF;
		hardware.light[GREENC].state = OFF;
		hardware.light[GREENR].state = OFF;
	}

	if (light == YELLOW)
	{
		hardware.light[YELLOWL].state = OFF;
		hardware.light[YELLOWC].state = OFF;
		hardware.light[YELLOWR].state = OFF;
	}

	if (light == BLUE)
	{
		hardware.light[BLUEL].state = OFF;
		hardware.light[BLUEC].state = OFF;
		hardware.light[BLUER].state = OFF;
	}
}